--[[
     < ArenaPlates v1.0 >
______________________________
 Author: Spyro
Testers: Woundman, Doomsen
License: All Rights Reserved
Contact: Spyrö  @ ArenaJunkies
         Spyro  @ WowInterface
         Spyro_ @ Curse/WowAce
¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯
]]

local AddonName, _ = ...

-- Upvalues
local _G, pairs, unpack, select, format, tonumber, strupper, strsub, strfind, hooksecurefunc, CreateFrame, WorldFrame, InterfaceOptionsFrame, InterfaceOptions_AddCategory
    = _G, pairs, unpack, select, format, tonumber, strupper, strsub, strfind, hooksecurefunc, CreateFrame, WorldFrame, InterfaceOptionsFrame, InterfaceOptions_AddCategory
local IsInInstance, UnitExists, UnitHealthMax, GetUnitName, UnitAffectingCombat, GetNumArenaOpponents, IsActiveBattlefieldArena, InterfaceOptionsFrame_OpenToCategory
    = IsInInstance, UnitExists, UnitHealthMax, GetUnitName, UnitAffectingCombat, GetNumArenaOpponents, IsActiveBattlefieldArena, InterfaceOptionsFrame_OpenToCategory

-- Addon's main frame
local Addon = CreateFrame("Frame")
      Addon:SetFrameStrata("LOW")

-- Misc vars
local UI -- Stores all the configuration user interface
local DB -- Local reference to the saved variable (ArenaPlatesDB)
local Number = {} -- FontStrings for the ArenaID numbers
local NumChildren = -1 -- WorldFrame child counter

-- Combat tracker vars
local VisiblePlates = {} -- Stores references to the visible nameplates that we want to check its combat status (indexed by UnitID)
local NumVisiblePlates = 0 -- Counter for the number of elements on the table VisiblePlates
local CombatChecker = CreateFrame("Frame") -- Frame whose OnUpdate will check the combat status of the nameplates on VisiblePlates
      CombatChecker.LastUpdate = 0

-- Default values of configuration
local Defaults = {
  FontSize       = 14,    -- Font size for the ArenaID numbers
  OpaqueMode     = true,  -- Show numbers always 100% opaque, even for non-targets
  HideNames      = false, -- Hide nameplate unitname
  CheckPrecision = 0.2,   -- Check combat every this seconds
  CombatColoring = true,  -- Change color of nameplates out of combat
  NoCombatColor  = { R = 0, G = 1, B = 0 } -- Vertex color for nameplates out of combat (applied on border texture)
}

-- Event registration
Addon:SetScript("OnEvent", function(self, event, ...) return self[event](self, ...) end)
Addon:RegisterEvent("ADDON_LOADED")
Addon:RegisterEvent("PLAYER_LOGIN")
Addon:RegisterEvent("PLAYER_ENTERING_WORLD")

-- CreateFakePlate()
-- Creates a nameplate like Blizzard ones, used for the example nameplates on the configuration UI.
-- > Class: Class of the hipotetical enemy.
-- > Name: Name of the hipotetical enemy.
-- < Plate: Reference to the created nameplate.
local function CreateFakePlate(Class, Name)
  -- Main frame
  local Plate = CreateFrame("Frame")
  Plate:SetSize(140.98, 39.16)
  Plate:SetFrameLevel(2)

  -- Health bar
  local Color = (CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS)[strupper(Class)]
  Plate.Healthbar = CreateFrame("StatusBar", nil, Plate)
  Plate.Healthbar:SetFrameLevel(1)
  Plate.Healthbar:SetStatusBarTexture("Interface/TargetingFrame/UI-TargetingFrame-BarFill")
  Plate.Healthbar:SetStatusBarColor(Color.r, Color.g, Color.b) -- Class color
  Plate.Healthbar:SetMinMaxValues(0, 1000)
  Plate.Healthbar:SetValue(1000)
  Plate.Healthbar:SetSize(104.75, 11)
  Plate.Healthbar:SetPoint("BOTTOMLEFT", Plate, "BOTTOMLEFT", 5.21, 4.89)

  -- Border
  Plate.Border = Plate:CreateTexture()
  Plate.Border:SetTexture("Interface/Tooltips/Nameplate-Border")
  Plate.Border:SetSize(264.98, 39.16)
  Plate.Border:SetPoint("TOPLEFT", Plate)

  -- Name
  Plate.Name = Plate:CreateFontString()
  Plate.Name:SetFont("Fonts/FRIZQT__.TTF", 12)
  Plate.Name:SetTextColor(1, 1, 1, 1)
  Plate.Name:SetShadowColor(0, 0, 0, 1)
  Plate.Name:SetShadowOffset(1.56, -1.56)
  Plate.Name:SetText(Name)
  Plate.Name:SetPoint("BOTTOM", Plate, "CENTER", 0, 0)

  -- Level
  Plate.Level = Plate:CreateFontString()
  Plate.Level:SetTextColor(1, 1, 0, 1)
  Plate.Level:SetFont("Fonts/FRIZQT__.TTF", 14, "OUTLINE")
  Plate.Level:SetPoint("CENTER", Plate, "BOTTOMRIGHT", -16.49, 10.57)
  Plate.Level:SetText("100")

  return Plate
end

-- CreateCheckButton()
-- Creates a checkbox.
-- > Parent: Checkbutton's parent frame.
-- > Checked: Bool that indicates if it should be checked by default or not.
-- > Label: Text on right of the checkbox.
-- > Tooltip: Tooltip that will appear while mouseovering.
-- > OnClickFunc: Reference to a function that will be executed when clicking this checkbox.
-- < CheckButton: Reference to the created check button.
local function CreateCheckButton(Parent, Checked, Label, Tooltip, OnClickFunc)
  local CheckButton = CreateFrame("CheckButton", nil, Parent, "ChatConfigCheckButtonTemplate")

  CheckButton.Label = select(6, CheckButton:GetRegions())
  CheckButton.Label:SetText(Label)
  CheckButton.Label:SetPoint("LEFT", CheckButton.Label:GetParent(), "RIGHT", 2, 0)
  CheckButton.tooltip = Tooltip
  CheckButton:SetChecked(Checked)
  CheckButton:SetScript("OnClick", function(self) OnClickFunc(self) end)

  return CheckButton
end

-- CreateSlider()
-- Creates a slider.
-- > Parent: Slider's parent frame.
-- > Label: Text on top of the slider.
-- > Tooltip: Tooltip that will appear while mouseovering the slider.
-- > Width: Width of the slider bar.
-- > Min: Minimal selectable value.
-- > Max: Maximum selectable value.
-- > Step: Difference between one value and the next.
-- > DefaultValue: Default selected value.
-- > OnValueChangedFunc: Reference to a function that will be executed when selecting values on the slider.
-- < Slider: Reference to the created slider.
local function CreateSlider(Parent, Label, Tooltip, Width, Min, Max, Step, DefaultValue, OnValueChangedFunc)
  local Slider = CreateFrame("Slider", nil, Parent, "OptionsSliderTemplate")

  -- Creating references to the slider's regions
  Slider.Label = select(10, Slider:GetRegions())
  Slider.Low   = select(11, Slider:GetRegions())
  Slider.High  = select(12, Slider:GetRegions())

  Slider.Label:SetText(Label)
  Slider.Low:SetText(Min)
  Slider.High:SetText(Max)
  Slider.tooltipText = Tooltip
  Slider:SetWidth(Width)
  Slider:SetMinMaxValues(Min, Max)
  Slider:SetObeyStepOnDrag(true)
  --Slider:SetStepsPerPage(1)
  Slider:SetValueStep(Step)
  Slider:SetValue(DefaultValue)
  Slider:SetScript("OnValueChanged", function(self, value) OnValueChangedFunc(self, value) end)

  return Slider
end

-- CreateColorPicker()
-- Creates a color picker that shows the selected color.
-- > Parent: Parent frame.
-- > Label: Label.
local function CreateColorPicker(Parent, Label)
  local Picker = CreateFrame("Button", nil, Parent, "OptionsCheckButtonTemplate")
  local P = Picker
  P:SetHighlightTexture("Interface/Buttons/CheckButtonHilight")

  -- Callback
  P.ColorCallback = function(Restore)
    local NewR, NewG, NewB
    if Restore then NewR, NewG, NewB = unpack(Restore)
    else NewR, NewG, NewB = ColorPickerFrame:GetColorRGB() end

    P.Texture:SetTexture(NewR, NewG, NewB)
    DB.NoCombatColor.R, DB.NoCombatColor.G, DB.NoCombatColor.B = NewR, NewG, NewB -- Saving changes
    if DB.CombatColoring then UI.Example.Plate[3].Border:SetVertexColor(NewR, NewG, NewB) end -- Updating the color on the example nameplate
  end

  -- OnClick
  P:SetScript("OnClick", function(Self)
    ColorPickerFrame.opacity  = 1
    ColorPickerFrame.hasOpacity = false
    ColorPickerFrame.previousValues = { DB.NoCombatColor.R, DB.NoCombatColor.G, DB.NoCombatColor.B }
    ColorPickerFrame.func = Self.ColorCallback
    ColorPickerFrame.opacityFunc = Self.ColorCallback
    ColorPickerFrame.cancelFunc = Self.ColorCallback
    ColorPickerFrame:Hide()
    ColorPickerFrame:Show() -- Triggering OnShow
    ColorPickerFrame:SetColorRGB(DB.NoCombatColor.R, DB.NoCombatColor.G, DB.NoCombatColor.B) -- Loading the saved color
  end)

  -- Selected color texture
  P.Texture = P:CreateTexture(nil, "OVERLAY")
  P.Texture:SetTexture("Interface/ChatFrame/ChatFrameColorSwatch")
  P.Texture:SetTexture(DB.NoCombatColor.R, DB.NoCombatColor.G, DB.NoCombatColor.B)
  P.Texture:SetPoint("CENTER")
  P.Texture:SetHeight(P:GetHeight() / 2)
  P.Texture:SetWidth(P:GetWidth() / 2 + 2)

  -- Label
  P.Label = P:CreateFontString(nil, nil, "GameFontNormal")
  P.Label:SetPoint("LEFT", P, "RIGHT")
  P.Label:SetText(Label)
  P.Label:SetTextColor(1, 1, 1)

  return P
end

-- CreateConfigUI()
-- Creates the configuration UI panel on Interface->Addons->ArenaPlates.
local function CreateConfigUI()
  local Label, Tooltip -- Temporal strings for the creation calls

  -- Creating new panel for the addon
  UI = CreateFrame("Frame", nil, InterfaceOptionsFrame)
  UI.name = "ArenaPlates"
  InterfaceOptions_AddCategory(UI)

  -- Frame holders for the example nameplates
  UI.Example = CreateFrame("Frame") -- Parent for the example nameplates
  UI.Example:SetFrameStrata("DIALOG")
  UI.Example:SetFrameLevel(1)
  UI.Example:Hide()
  UI.Example.Plate = {}
  UI.Example.OpaqueParent = CreateFrame("Frame", nil, UI.Example) -- Parent for the ArenaID numbers when OpaqueMode is active
  UI.Example.OpaqueParent:SetFrameStrata("DIALOG")
  UI.Example.OpaqueParent:SetFrameLevel(2)
  UI:HookScript("OnShow", function() UI.Example:Show() end)
  UI:HookScript("OnHide", function() UI.Example:Hide() end)

  -- Creating example enemy nameplates to see the addon effects on the fly
  local i = 1
  -- Class = ExampleName
  for Class, Name in pairs({ ["Rogue"] = "Arena1", ["Mage"] = "Arena2", ["Priest"] = "Arena3 (OOC)" }) do
    UI.Example.Plate[i] = CreateFakePlate(Class, Name)
    local Plate = UI.Example.Plate[i]
    Plate:SetParent(UI.Example)
    Plate.Name:SetShown(not DB.HideNames) -- HideNames option
    Plate.Level:SetText(i)
    if i ~= 1 then Plate:SetPoint("CENTER", UI.Example.Plate[i-1], "CENTER", 0, -33) end
    i = i + 1
  end

  -- Title
  UI.Title = UI:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
  UI.Title:SetText("ArenaPlates")
  UI.Title:SetPoint("TOPLEFT", 16, -16)

  -- HideNames option checkbox
  Label = "Hide unitnames"
  Tooltip = "Hides the unitnames to make nameplates simpler"
  UI.HideNamesCheck = CreateCheckButton(UI, DB.HideNames, Label, Tooltip, function(self)
    DB.HideNames = self:GetChecked() and true or false
    for i = 1, 3 do UI.Example.Plate[i].Name:SetShown(not DB.HideNames) end
  end)
  UI.HideNamesCheck:SetPoint("TOPLEFT", UI.Title, "BOTTOMLEFT", 0, -10)
  UI.Example.Plate[1]:SetPoint("LEFT", UI.HideNamesCheck.Label, "RIGHT", 70, 8)

  -- OpaqueMode checkbox
  Label = "Always opaque numbers"
  Tooltip = "Shows numbers always fully opaque (even for non-targets)"
  UI.OpaqueModeCheck = CreateCheckButton(UI, DB.OpaqueMode, Label, Tooltip, function(self)
    DB.OpaqueMode = self:GetChecked() and true or false
    for i = 1, 3 do UI.Example.Plate[i].Level:SetParent(DB.OpaqueMode and UI.Example.OpaqueParent or UI.Example.Plate[i]) end
  end)
  UI.OpaqueModeCheck:HookScript("OnEnter", function()
    UI.Example.Plate[1]:SetAlpha(0.5)
    UI.Example.Plate[3]:SetAlpha(0.5)
    UI.Example.Plate[1].Name:SetText("NoTarget")
    UI.Example.Plate[2].Name:SetText("Target")
    UI.Example.Plate[3].Name:SetText("NoTarget")
    for i = 1, 3 do UI.Example.Plate[i].Name:SetShown(true) end
  end)
  UI.OpaqueModeCheck:HookScript("OnLeave", function()
    UI.Example.Plate[1]:SetAlpha(1)
    UI.Example.Plate[3]:SetAlpha(1)
    UI.Example.Plate[1].Name:SetText("Arena1")
    UI.Example.Plate[2].Name:SetText("Arena2")
    UI.Example.Plate[3].Name:SetText("Arena3 (OOC)")
    for i = 1, 3 do UI.Example.Plate[i].Name:SetShown(not DB.HideNames) end
  end)
  UI.OpaqueModeCheck:SetPoint("TOPLEFT", UI.HideNamesCheck, "BOTTOMLEFT", 0, 0)

  -- CombatColoring checkbox
  Label = "Combat coloring"
  Tooltip = "Changes the color of nameplates out of combat"
  UI.CombatColoringCheck = CreateCheckButton(UI, DB.CombatColoring, Label, Tooltip, function(self)
    DB.CombatColoring = self:GetChecked() and true or false
    -- Updating the color on the example nameplate
    UI.Example.Plate[3].Border:SetVertexColor(1, 1, 1)
    if DB.CombatColoring then UI.Example.Plate[3].Border:SetVertexColor(DB.NoCombatColor.R, DB.NoCombatColor.G, DB.NoCombatColor.B) end
  end)
  UI.CombatColoringCheck:HookScript("OnEnter", function()
    UI.Example.Plate[1].Name:SetText("Combat")
    UI.Example.Plate[2].Name:SetText("Combat")
    UI.Example.Plate[3].Name:SetText("NoCombat")
    for i = 1, 3 do UI.Example.Plate[i].Name:SetShown(true) end
  end)
  UI.CombatColoringCheck:HookScript("OnLeave", UI.OpaqueModeCheck:GetScript("OnLeave"))
  UI.CombatColoringCheck:SetPoint("TOPLEFT", UI.OpaqueModeCheck, "BOTTOMLEFT", 0, 0)

  -- Color picker frame to select the color for nameplates out of combat
  Label = "Out of combat color"
  UI.NoCombatColorPicker = CreateColorPicker(UI, Label)
  UI.NoCombatColorPicker:SetPoint("TOPLEFT", UI.CombatColoringCheck, "BOTTOMLEFT", 0, 0)

  -- Font size slider
  Label = "ArenaID font size: "..DB.FontSize
  Tooltip = nil
  UI.FontSizeSlider = CreateSlider(UI, Label, Tooltip, 155, 12, 23, 1, DB.FontSize, function(self, value)
    DB.FontSize = value
    self.Label:SetText("ArenaID font size: "..DB.FontSize)
    for i = 1, 3 do UI.Example.Plate[i].Level:SetFont("Fonts/FRIZQT__.TTF", DB.FontSize, "OUTLINE") end
    -- Applying size change to already created ArenaID number FontStrings
    for K, N in pairs(Number) do N:SetFont("Fonts/FRIZQT__.TTF", DB.FontSize, "OUTLINE") end
  end)
  UI.FontSizeSlider:SetPoint("TOPLEFT", UI.NoCombatColorPicker, "BOTTOMLEFT", 5, -30)

  -- CheckPrecision slider
  Label = "Check precision: "..DB.CheckPrecision.." sec"
  Tooltip = "Checks the combat status every this seconds. The lower the number the higher the precision (but also the CPU use)"
  UI.CheckPrecisionSlider = CreateSlider(UI, Label, Tooltip, 155, 0.1, 0.5, 0.1, DB.CheckPrecision, function(self, value)
    DB.CheckPrecision = tonumber(format("%.1f", value))
    self.Label:SetText("Check precision: "..DB.CheckPrecision.." sec")
  end)
  UI.CheckPrecisionSlider:SetPoint("TOPLEFT", UI.FontSizeSlider, "BOTTOMLEFT", 0, -40)

  -- Reset to Default button
  local Reset = CreateFrame("Button", nil, UI, "UIPanelButtonTemplate")
  Reset:SetSize(120, 25)
  Reset:SetText("Reset to Default")
  Reset:SetScript("OnClick", function() -- Rolling all options back to default values
    if UI.HideNamesCheck:GetChecked() then UI.HideNamesCheck:Click() end
    if not UI.OpaqueModeCheck:GetChecked() then UI.OpaqueModeCheck:Click() end
    if not UI.CombatColoringCheck:GetChecked() then UI.CombatColoringCheck:Click() end
    ColorPickerFrame:SetColorRGB(0, 1, 0)
    UI.NoCombatColorPicker.ColorCallback()
    UI.FontSizeSlider:SetValue(14)
    UI.CheckPrecisionSlider:SetValue(0.2)
  end)
  Reset:SetPoint("TOPLEFT", UI.CheckPrecisionSlider, "BOTTOMLEFT", 0, -30)

  -- Loading saved options to the example nameplates
  for i = 1, 3 do
    UI.Example.Plate[i].Name:SetShown(not DB.HideNames)
    UI.Example.Plate[i].Level:SetFont("Fonts/FRIZQT__.TTF", DB.FontSize, "OUTLINE")
    UI.Example.Plate[i].Level:SetParent(DB.OpaqueMode and UI.Example.OpaqueParent or UI.Example.Plate[i])
  end
  UI.Example.Plate[3].Border:SetVertexColor(1, 1, 1)
  if DB.CombatColoring then UI.Example.Plate[3].Border:SetVertexColor(DB.NoCombatColor.R, DB.NoCombatColor.G, DB.NoCombatColor.B) end
end

-- CopyDefaults()
-- Copies the default settings on the saved variable in the empty fields.
-- > Source: Table with the default settings.
-- > Destination: Table with the current settings.
-- < Destination: Reference to the table with the current settings.
local function CopyDefaults(Source, Destination)
  if not Source then return {} end
  if not Destination then Destination = {} end

  for k, v in pairs(Source) do
    if type(v) == "table" then Destination[k] = CopyDefaults(v, Destination[k])
    elseif type(v) ~= type(Destination[k]) then Destination[k] = v end
  end

  return Destination
end

-- PutNumber()
-- Replaces the level number of a nameplate with an ArenaID number.
-- > i: Number digit.
-- > Plate: Nameplate reference.
local function PutNumber(i, Plate)
  local N

  if not Number[i] then -- Creating a new number
    Number[i] = Addon:CreateFontString()
    N = Number[i]
    N:SetFont("Fonts/FRIZQT__.TTF", DB.FontSize, "OUTLINE")
    N:SetTextColor(1, 1, 0, 1)
    N:SetDrawLayer("OVERLAY")
    N:SetText(i)
  end

  N = Number[i]
  Plate.NameContainer:SetShown(not DB.HideNames) -- Option for hiding the unitname
  Plate.ArtContainer.LevelText:Hide() -- Hiding the level text, coz the ArenaID number will be here
  Plate.ArenaID = N -- Storing a reference to the ArenaID number, for later use in NAMEPLATE_HIDE
  N:SetPoint("CENTER", Plate, "BOTTOMRIGHT", -16.49, 10.57)

  -- If OpaqueMode (always 100% opacity) is not wanted, configuring the number to inherit the nameplate opacity
  if not DB.OpaqueMode then
    N:SetParent(Plate.ArtContainer.LevelText:GetParent())
    N:SetDrawLayer(Plate.ArtContainer.LevelText:GetDrawLayer())
  end

  N:Show()
end

-- CombatCheckerFunc()
-- OnUpdate function for the CombatChecker frame.
-- Checks the combat status of visible nameplates that pertain to arena enemies, and changes the border color based on it.
-- > self: Reference to the CombatChecker frame.
-- > Elapsed: Time since the last OnUpdate cycle.
local function CombatCheckerFunc(self, Elapsed)
  -- OnUpdate time check
  self.LastUpdate = self.LastUpdate + Elapsed
  if self.LastUpdate < DB.CheckPrecision then return end
  self.LastUpdate = 0

  -- The time has passed, checking combat status on visible nameplates
  for UnitID, Plate in pairs(VisiblePlates) do
    if UnitAffectingCombat(UnitID) then Plate.ArtContainer.Border:SetVertexColor(1, 1, 1) -- Default color
    else Plate.ArtContainer.Border:SetVertexColor(DB.NoCombatColor.R, DB.NoCombatColor.G, DB.NoCombatColor.B) end -- OOC Color
  end
end

-- InArena()
-- Returns if the player is on an arena match.
local function InArena()
  return IsActiveBattlefieldArena() or select(2, IsInInstance()) == "arena"
end

-- GetArenaUnitID()
-- Returns the Arena UnitID of a nameplate.
-- > Plate: Nameplate reference.
-- < UnitID (nil if not found).
local function GetArenaUnitID(Plate)
  -- Iterating thru all the arena enemies and their pets
  for i = 1, GetNumArenaOpponents() do
    for _, UnitID in pairs({"arena"..i, "arenapet"..i}) do
      if GetUnitName(UnitID) == Plate.NameContainer.NameText:GetText() then return UnitID end
    end
  end

  return nil
end

-- Event NAMEPLATE_SHOW
-- Fires when a nameplate appears on screen.
-- > Plate: Nameplate reference.
local function NAMEPLATE_SHOW(Plate)
  local UnitID = GetArenaUnitID(Plate) -- UnitID of the owner of this nameplate
  if not UnitID then return end -- Doesn't have an UnitID
  Plate.UnitID = UnitID -- Storing nameplate's UnitID (for future use in NAMEPLATE_HIDE)

  -- If it's an enemy player, put his ArenaID number on his nameplate where the level number is
  if strfind(UnitID, "^arena%d") then PutNumber(strsub(UnitID, 6), Plate) end

  -- If we want to color by combat status, add this nameplate to the combat checker process
  if DB.CombatColoring then
    if NumVisiblePlates == 0 then CombatChecker:SetScript("OnUpdate", CombatCheckerFunc) end -- Enabling combat tracker
    VisiblePlates[UnitID] = Plate
    NumVisiblePlates = NumVisiblePlates + 1
  end
end

-- Event NAMEPLATE_HIDE
-- Fires when a nameplate disappears from screen.
-- > Plate: Nameplate reference.
local function NAMEPLATE_HIDE(Plate)
  if not Plate.UnitID then return end -- Doesn't have an UnitID

  -- Removing from the combat tracker
  if DB.CombatColoring then
    VisiblePlates[Plate.UnitID] = nil
    NumVisiblePlates = NumVisiblePlates - 1
    if NumVisiblePlates == 0 then CombatChecker:SetScript("OnUpdate", nil) end -- Disabling combat tracker
    Plate.ArtContainer.Border:SetVertexColor(1, 1, 1) -- Restoring default color
  end
  Plate.UnitID = nil

  -- If it has an ArenaID number attached, free it
  if Plate.ArenaID then
    Plate.ArenaID:Hide()
    Plate.ArenaID:ClearAllPoints()
    Plate.ArenaID:SetParent(Addon)
    Plate.ArenaID = nil
    Plate.NameContainer:Show()
    Plate.ArtContainer.LevelText:Show()
  end
end

-- SetScriptHook()
-- Function to re-hook the script handlers OnShow/OnHide on nameplates when other addons destroy them using SetScript() instead of HookScript().
-- > Plate: Nameplate reference.
-- > Handler: Script handler used on the SetScript() call.
-- > Func: Hooked function.
local function SetScriptHook(Plate, Handler, Func)
  if Handler == "OnShow" then Plate:HookScript("OnShow", NAMEPLATE_SHOW)
  elseif Handler == "OnHide" then Plate:HookScript("OnHide", NAMEPLATE_HIDE) end
end

-- PlateProcess()
-- Process the WorldFrame children whenever a new children it's created, to catch nameplates.
local function PlateProcess()
  if WorldFrame:GetNumChildren() == NumChildren then return end
  NumChildren = WorldFrame:GetNumChildren()

  -- Iterating thru all WorldFrame's children to catch nameplates
  for _, Plate in pairs({ WorldFrame:GetChildren() }) do
    if not Plate.ArenaPlates and Plate.ArtContainer then -- It's a nameplate not seen before
      Plate.ArenaPlates = true -- Mark as seen to ignore it in future iterations

      -- Script handlers
      Plate:HookScript("OnShow", NAMEPLATE_SHOW) -- Event to trigger when the nameplate appears on screen
      Plate:HookScript("OnHide", NAMEPLATE_HIDE) -- Event to trigger when the nameplate disappears from screen
      if Plate:IsVisible() then NAMEPLATE_SHOW(Plate) end -- If it's already visible trigger for the first time

      -- Hooking the use of SetScript() on nameplates by other addons, so they don't fuck our hooks :-)
      hooksecurefunc(Plate, "SetScript", SetScriptHook)
    end
  end
end

-- Event ADDON_LOADED
-- Fires when an addon and its saved variables are loaded.
function Addon:ADDON_LOADED(Name)
  if Name ~= "ArenaPlates" then return end
  Addon:UnregisterEvent("ADDON_LOADED")

  if not ArenaPlatesDB then ArenaPlatesDB = {} end -- No saved variable, creating a new one
  DB = ArenaPlatesDB -- Local reference for faster access
  CopyDefaults(Defaults, DB)

  CreateConfigUI()
end

-- Event PLAYER_LOGIN
-- Fires after PLAYER_ENTERING_WORLD after logging in and after /reloadui.
function Addon:PLAYER_LOGIN()
  -- Enabling nameplate processing if we are in arena after logging in (like after a DC)
  Addon:SetScript("OnUpdate", InArena() and PlateProcess or nil)
end

-- Event PLAYER_ENTERING_WORLD
-- Fires after a loading screen.
function Addon:PLAYER_ENTERING_WORLD()
  -- If it's an arena, enabling nameplate processing, disabling it otherwise
  Addon:SetScript("OnUpdate", InArena() and PlateProcess or nil)
end

-- Command /arenaplates
-- Opens the configuration UI of the addon.
SLASH_ARENAPLATES1 = "/arenaplates"
SlashCmdList["ARENAPLATES"] = function()
  -- Calling 2 times: Workaround for the bug of this function that doesn't open the correct panel the first time
  InterfaceOptionsFrame_OpenToCategory(UI)
  InterfaceOptionsFrame_OpenToCategory(UI)
end